const moment = require('moment');
const trunc2 = require('../functions/Trunc2');
const incMonth = require('../functions/IncMonth');
const constructQuery = require('../functions/ConstructQuery');
const oracleService = require('../service/OracleService');
const GeraNovoCodigo = require('./NovoCodigoModule');
const acmInsMovCaixaPag = require('../model/acmInsMovCaixaPagModel');
const ClientDataSet = require('../components/ClientDataSet');
require('moment/locale/pt-br');

const Now = function () {
  return moment().format('Y-MM-DD HH:mm:ss');
};

const DateFormat = function (data) {
  data = moment(data).utc().format('Y-MM-DD HH:mm:ss');
  return data;
};

async function TransfDados(
  DataSetOrigem,
  obj = false,
  cdsVendaTemp = [],
  cdsMovCaixaTemp = [],
  cdsPedidoFosTemp = [],
  qtdePedidoFosMSDE = 0,
) {
  // Monta query da tabela e coloca no datasetdestino
  const data = obj === false ? JSON.parse(DataSetOrigem.data) : DataSetOrigem.data;
  const { tabela } = DataSetOrigem;
  let qtdePedidoFos = qtdePedidoFosMSDE;

  // Busca dados da tabela
  const { insertQuery, updateQuery, fields } = await constructQuery(tabela);

  if (data.length > 0) {
    for (const item of data) {
      // Inicia insert
      // Itera sobre os campos

      try {
        const params = {};
        const missedFields = [];
        const mapKeys = Object.keys(item);

        try {
          for (const field of fields) {
            if (mapKeys.indexOf(field.name) >= 0) {
              if (field.dbTypeName === 'DATE') {
                if (tabela === 'MOVBENEFICIOCONS') {
                  if (field.name === 'DTULTATU') {
                    params[field.name] = DateFormat(new Date());
                  } else {
                    params[field.name] = item[field.name] ? DateFormat(item[field.name]) : null;
                  }
                } else {
                  params[field.name] = item[field.name] ? DateFormat(item[field.name]) : null;
                }
              } else if (field.dbTypeName === 'VARCHAR2') {
                if (field.name === 'NRSEQVENDA' && cdsVendaTemp instanceof ClientDataSet) {
                  if (item[field.name]) {
                    const vendaTemp = cdsVendaTemp.locate(['NRSEQANT'], [item[field.name]]);
                    params[field.name] = vendaTemp ? vendaTemp['NRSEQNEW'] : null;
                  } else {
                    params[field.name] = null;
                  }
                } else if (field.name === 'NRSEQUMOVI') {
                  if (tabela === 'MOVCAIXA') {
                    const nrSequMovi = await GeraNovoCodigo.prxMovCaixa(
                      item['CDFILIAL'],
                      item['CDCAIXA'],
                      item['DTABERCAIX'],
                    );
                    params[field.name] = nrSequMovi;
                    const movCaixa = {
                      NRSEQANT: item[field.name],
                      NRSEQNEW: nrSequMovi,
                      CDFILIAL: item['CDFILIAL'],
                      CDCAIXA: item['CDCAIXA'],
                      NRSEQVENDA: item['NRSEQVENDA'],
                      DTABERCAIX: item['DTABERCAIX'],
                    };
                    cdsMovCaixaTemp.data.push(movCaixa);
                  } else if (item[field.name]) {
                    const cdsMovCaixaTempFilterd = cdsMovCaixaTemp.data.filter((element) => {
                      return (
                        element['CDFILIAL'] === item['CDFILIAL'] &&
                        element['CDCAIXA'] === item['CDCAIXA'] &&
                        element['NRSEQVENDA'] === item['NRSEQVENDA'] &&
                        element['DTABERCAIX'] === item['DTABERCAIX']
                      );
                    });

                    params[field.name] = cdsMovCaixaTempFilterd.length
                      ? cdsMovCaixaTempFilterd['NRSEQNEW']
                      : null;
                  } else {
                    params[field.name] = null;
                  }
                } else if (field.name === 'NRSEQMOVCLI') {
                  const nrSeqMovCli = await GeraNovoCodigo.prxMoviClie(item['CDCLIENTE']);
                  params[field.name] = nrSeqMovCli;
                } else if (field.name === 'IDIMPREDUCAO' || field.name === 'IDIMPMOVCAIXA') {
                  params[field.name] = 'S';
                } else if (field.name === 'IDIMPTMOV') {
                  params[field.name] = 'N';
                } else if (field.name === 'NRSEQBENECONS') {
                  params[field.name] = await GeraNovoCodigo.prxMovBeneficioCons();
                } else if (field.name === 'NRPEDIDOFOS') {
                  if (tabela === 'PEDIDOFOS') {
                    const nrPedidoFos = await GeraNovoCodigo.prxPedidoFos(
                      item['CDFILIAL'],
                      qtdePedidoFos,
                    );
                    const pedidoFosTemp = {
                      NRSEQANT: item[field.name],
                      NRSEQNEW: nrPedidoFos,
                    };
                    cdsPedidoFosTemp.data.push(pedidoFosTemp);
                    params[field.name] = nrPedidoFos;
                    qtdePedidoFos = 1;
                  } else if (item[field.name]) {
                    const pedidoFos = cdsPedidoFosTemp.locate(['NRSEQANT'], item[field.name]);
                    params[field.name] = pedidoFos ? pedidoFos['NRSEQNEW'] : null;
                  } else {
                    params[field.name] = null;
                  }
                } else {
                  params[field.name] = item[field.name];
                }
              } else if (field.dbTypeName === 'FLOAT') {
                params[field.name] = parseFloat(item[field.name]);
              } else {
                params[field.name] = item[field.name];
              }
            } else {
              missedFields.push(field.name);
            }
          }

          if (missedFields.length) {
            throw { error: 'true', message: 'missing fields on ' + tabela, fields: missedFields };
          } else {
            try {
              try {
                const execution = await oracleService.execute(insertQuery, params);

                if (!DataSetOrigem.autocommit || DataSetOrigem.autocommit !== 'false') {
                  await execution.connection.commit();
                }
              } catch (error) {
                console.log(error);
              }
            } catch (error) {
              console.log('Não foi possível inserir.');
            }
          }

          if (tabela === 'MOVCAIXA') {
            if (item['QTPARCRECEB'] && parseInt(item['QTPARCRECEB'], 10) > 1) {
              const qtParcReceb = parseInt(item['QTPARCRECEB'], 10);
              let dias = 1;
              let valParc = 0;

              while (dias <= qtParcReceb) {
                valParc += trunc2(item['VRMOVIVEND']) - qtParcReceb;
                const updParams = {
                  P_CDFILIAL: item['CDFILIAL'],
                  P_CDCAIXA: item['CDCAIXA'],
                  P_DTABERCAIX: item['DTABERCAIX'],
                  P_NRSEQUMOVI: item['NRSEQUMOVI'],
                  P_NRSEQVENDA: item['NRSEQVENDA'],
                  P_NRSEQMOVPAG: dias,
                  P_VRPARCRECEB: valParc,
                };

                if (!(dias + 1) <= qtParcReceb) {
                  if (valParc < parseFloat(item['VRMOVIVEND'])) {
                    valParc =
                      trunc2(item['VRMOVIVEND']) / qtParcReceb +
                      parseFloat(item['VRMOVIVEND']) -
                      trunc2(valParc);
                  }
                }

                const dtVencimento = incMonth(new Date(item['DTMOVIMCAIXA']), dias);
                updParams['P_DTVENCIMENTO'] = DateFormat(dtVencimento);
                await acmInsMovCaixaPag.get(updParams);
                dias += 1;
              }
            }
          }
        } catch (error) {
          console.log(error);
          throw error;
        }
      } catch (error) {
        if (error) {
          console.log(error);
          throw error;
        }
      }
    }
  }
}

module.exports = TransfDados;
