import { SocketEvent } from '@/socket/socket-event';
import { AlertService, Modal, ModalService } from '@zeedhi/common';
import { Config, Http, IDictionary, Metadata } from '@zeedhi/core';
import { TekGrid, TekGridColumn, ITekGridColumn, IFilterPropsComponent, TekMemoryDatasource } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';

export class MonitoringController {
  public socket?: Socket;

  public isVisibleModalColumn: boolean = false;

  private monitoringModal!: Modal;

  private cdfilial: any;

  private clickedAssConfig: any;

  private assConfig: any;

  private satColumnId: any;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  public gridBeforeApplyFilter() {
    const { datasource } = Metadata.getInstance<TekGrid>('monitoring-grid');
    const filter = (datasource as TekMemoryDatasource).dynamicFilter;
    localStorage.setItem('LAST_FILTER_MONITORING', JSON.stringify(filter));
  }

  public async gridMounted() {
    const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
    this.monitoringModal = await ModalService.createFromJson('monitoringModal', true);

    try {
      grid.datasource.loading = true;

      if (!this.socket) {
        this.socket = io(Config.endPoint);
      }

      const reqParams = `${process.env.VUE_APP_METADATA_END_POINT}parametrizacoes`;
      const columns = await Http.get(reqParams);
      this.addColumns(columns, grid);

      const reqMonitoring = `${process.env.VUE_APP_METADATA_END_POINT}monitoring`;
      const monitoring = await Http.get(reqMonitoring);

      const lastFilter = localStorage.getItem('LAST_FILTER_MONITORING');
      if (lastFilter) {
        const filter = JSON.parse(lastFilter);
        if (Object.keys(filter).length > 0) {
          const actualColumns = columns.data.data.filter((element: any) => element.ATIVO === 'S')
                                                    .map((element: any) => element.ID);
          Object.keys(filter).forEach((item: any) => {
            if (!actualColumns.includes(item)) {
              delete filter[item];
            }
          });
          (grid.datasource as TekMemoryDatasource).dynamicFilter = filter;
        }
      }

      grid.datasource.updateData(monitoring.data.data);
      grid.datasource.loading = false;

      this.socketEmit(columns);
      this.socketListener(columns, grid);
    } catch (error) {
      grid.datasource.loading = false;
      this.showAlert(error, 'error');
      console.log(error);
    }
  }

  private socketEmit(columns: any) {
    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);

    setInterval(() => {
      this.socket?.emit('ping', { id: this.socket?.id });
    }, 20000);

    columns.data.data.forEach((element: any) => {
      if (element.ATIVO === 'S') {
        this.socket?.emit(element.ID, { interval: element.INTERVALO * 1000 });
      }
    });
  }

  private socketListener(columns: any, grid: TekGrid) {
    columns.data.data.forEach((element: any) => {
      if (element.ATIVO === 'S') {
        this.socket?.on(element.ID, async (response: any) => {
          grid.datasource.updateData(response);
        });
      }
    });

    this.socket?.on('login', async (response: any) => {
      if (response.text === 'UNIDADE CONECTADA' || response.text === 'UNIDADE DESCONECTADA') {
        grid.datasource.updateData(response.message.data);
      }
    });

    this.socket?.on('disconnect', () => {
      this.showAlert('A página será recarregada para restabelecer a conexão.', 'error');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });
  }

  private addColumns(columns: any, grid: TekGrid) {
    this.assConfig = columns.data.data;
    columns.data.data.forEach((element: any) => {
      if (element.ATIVO === 'S') {
        if (element.NOME === 'SAT') {
          this.satColumnId = element.ID;
        }
        const newColumn = this.getNewColumn(element, grid);
        if (element.NOME !== 'SAT') {
          newColumn.filterProps = this.getNewColumnFilterProps(element);
        }
        grid.columns.push(newColumn);
      }
    });
  }

  private getNewColumn(element: any, grid: TekGrid) {
    const newColumn: ITekGridColumn = {
      name: element.ID,
      label: element.NOME,
      type: 'action',
      maxWidth: '100px',
      children: [
        {
          name: `button_${element.ID}`,
          component: 'ZdButton',
          small: true,
          icon: true,
          iconName: 'mdi-brightness-1',
          conditions: {
            color: this.getButtonColor.bind(this),
          },
          events: {
            click: this.buttonClickMonitoring.bind(this),
          },
        },
      ],
    };
    return new TekGridColumn(newColumn, grid);
  }

  private getNewColumnFilterProps(element: any) {
    const filterProps: IFilterPropsComponent = {
      name: element.ID,
      label: element.NOME,
      relation: 'AND',
      operation: 'EQUALS',
      grid: {
        sm: 3,
      },
      component: 'ZdSelect',
      autocomplete: false,
      dataText: ['text'],
      dataValue: 'value',
      datasource: {
        data: [
            {
              value: 'S',
              text: 'Sucesso',
            },
            {
              value: 'N',
              text: 'Erro',
            },
        ],
      },
    };
    return filterProps;
  }

  public closeModal() {
    this.monitoringModal.hide();
  }

  public modalGridMounted() {
    const modalGrid = Metadata.getInstance<TekGrid>('monitoring-modal-grid');
    modalGrid.datasource.uniqueKey = 'NAME';
    const column = <TekGridColumn> modalGrid.getColumn('NAME');
    modalGrid.datasource.loading = true;
    this.socket?.emit(`${this.clickedAssConfig.ID}_detail`, { CDFILIAL: this.cdfilial });
    this.socket?.on(`${this.clickedAssConfig.ID}_detail`, async (response: any) => {
      this.isVisibleModalColumn = false;
      if (response && response.rows && response.rows.length) {
        if (this.clickedAssConfig.NOME === 'SAT') {
          this.isVisibleModalColumn = true;
        } else if (response.metaData) {
          response.metaData.forEach((element: any, key: any) => {
            const newColumn = new TekGridColumn({ ...column }, modalGrid);
            newColumn.name = key;
            newColumn.label = element.name;
            newColumn.isVisible = true;
            modalGrid.columns.push(newColumn);
          });
        } else {
          Object.keys(response.rows[0]).forEach((element: any) => {
            let newColumn;
            if (this.clickedAssConfig.TIPO === 'S' && element === 'STATUS') {
              newColumn = this.getServicesStatusColumn(element, modalGrid);
              modalGrid.datasource.uniqueKey = 'SERVICO';
            } else {
              newColumn = new TekGridColumn({ ...column }, modalGrid);
              newColumn.name = element;
              newColumn.label = element;
              newColumn.isVisible = true;
            }
            modalGrid.columns.push(newColumn);
          });
        }
        modalGrid.datasource.updateData(response.rows);
      }
      modalGrid.datasource.loading = false;
    });
  }

  private getServicesStatusColumn(element: any, grid: TekGrid) {
    const newColumn: ITekGridColumn = {
      name: element,
      label: element,
      type: 'action',
      maxWidth: '100px',
      fixed: true,
      children: [
        {
          name: `button_${element}`,
          component: 'ZdButton',
          small: true,
          icon: true,
          iconName: 'mdi-brightness-1',
          conditions: {
            color: this.getServicesButtonColor.bind(this),
          },
        },
      ],
    };
    return new TekGridColumn(newColumn, grid);
  }

  public gridBeforeDestroy() {
    this.monitoringModal?.destroy();
  }

  public getButtonColor({ column, row }: IDictionary<any>) {
    let cor;
    if (column.name === 'websocket') {
      cor = (row.situacao === 'Ativo') ? 'green' : 'red';
    } else if (this.satColumnId && column.name === this.satColumnId && row.sat === 'N') {
      cor = 'gray';
    } else {
      cor = row[column.name] === 'S' && row.situacao === 'Ativo' ? 'green' : 'red';
    }
    return cor;
  }

  public getSatButtonColor({ row }: IDictionary<any>) {
    return row.sat ? 'green' : 'red';
  }

  public getServicesButtonColor({ row }: IDictionary<any>) {
    return row.STATUS === 'S' ? 'green' : 'red';
  }

  public buttonClickMonitoring(monitor:any) {
    if (monitor.component.color === 'green') {
      this.showAlert('Tudo certo com esse monitoramento!', 'success');
    } else if (monitor.column.name === this.satColumnId && monitor.row.sat === 'N') {
      this.showAlert('Essa filial não utiliza modelo fiscal SAT', '#e68129');
    } else if (monitor.row.situacao === 'Ativo') {
      this.cdfilial = monitor.row.id;
      this.clickedAssConfig = this.assConfig.find((element: any) => element.ID === monitor.column.name);
      this.monitoringModal.children[1].title = this.clickedAssConfig.DESCRICAO;
      this.monitoringModal.show();
    } else {
      this.showAlert('A conexão deve estar ativa para visualizar os monitoramentos', '#e68129');
    }
  }

  private showAlert(message: any, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }
}
